/* ***********************************************************/
/* Copyright (c)2007 by Progress Software Corporation        */
/*                                                           */
/* All rights reserved.  No part of this program or document */
/* may be  reproduced in  any form  or by  any means without */
/* permission in writing from Progress Software Corporation. */
/*************************************************************/

/* this file handles compiles for OpenEdge version 10.1B and
*  higher using the new language features for a compile error
*  messages.
*/


/* compile.p - Eclipse compile procedure */
DEFINE INPUT PARAMETER cPrm            AS CHARACTER NO-UNDO.
DEFINE SHARED VARIABLE cRoot    AS CHARACTER.

DEFINE SHARED VARIABLE cLinkedResources  AS CHARACTER.
DEFINE NEW SHARED VARIABLE XCodeKey      AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE XRefFile      AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE CodeListFile  AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE DebugListFile AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE RCodePath     AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE SourceFileFull     AS CHARACTER   NO-UNDO INIT "".
DEFINE NEW SHARED VARIABLE ErrorLogFile  AS CHARACTER   NO-UNDO.

DEFINE NEW SHARED STREAM WEBSTREAM.  /** Allow WebSpeed compile */
DEFINE STREAM CMPLSTREAM.
DEFINE STREAM s1.

DEFINE VARIABLE cRet         AS CHARACTER NO-UNDO.
DEFINE VARIABLE c1           AS CHARACTER NO-UNDO.

/** Compile variables **/
DEFINE VARIABLE cNewfile        AS CHARACTER   NO-UNDO INIT "".
DEFINE VARIABLE SourceFile      AS CHARACTER   NO-UNDO INIT "".
DEFINE VARIABLE cSpeedfile      AS CHARACTER   NO-UNDO INIT "".
DEFINE VARIABLE LiteralOptions  AS CHARACTER   NO-UNDO INIT "".
DEFINE VARIABLE CheckSyntax     AS LOGICAL     NO-UNDO INIT FALSE.
DEFINE VARIABLE SaveRCode       AS LOGICAL     NO-UNDO INIT FALSE.
DEFINE VARIABLE SaveXRef        AS LOGICAL     NO-UNDO INIT FALSE.
DEFINE VARIABLE CodeListing     AS LOGICAL     NO-UNDO INIT FALSE.
DEFINE VARIABLE DebugListing    AS LOGICAL     NO-UNDO INIT FALSE.
DEFINE VARIABLE SyntaxFile      AS CHARACTER   NO-UNDO INIT "". 
DEFINE VARIABLE CompileTempFile AS CHARACTER   NO-UNDO INIT "". 

DEFINE VARIABLE XRefXmlDirectory AS CHARACTER   NO-UNDO INIT "". 

DEFINE VARIABLE lHTMLMapped      AS LOGICAL     NO-UNDO INIT FALSE.

&GLOBAL-DEFINE COMMAND_SEPARATOR CHR(3)
&GLOBAL-DEFINE OPTION_SEPARATOR CHR(4)


RUN separateOptions.
 
 
IF checksyntax THEN DO:
	ASSIGN
		SaveRCode = false
		SourceFileFull = SyntaxFile.
END.
ELSE DO:
		SourceFileFull = SourceFile.
END.
ErrorLogFile = cLinkedResources + "/error.log".
CompileTempFile = cLinkedResources + "/../compile_tmp_" + STRING(ETIME) + ".p".

IF SourceFile MATCHES "*~~.html" OR SourceFile MATCHES "*~~.htm" THEN DO:
    DEFINE VARIABLE idx AS INTEGER NO-UNDO.

    idx = R-INDEX(SourceFile, ".").    
    ASSIGN cNewfile = SUBSTRING(SourceFile, 1, idx) + "w".
    
    FILE-INFO:FILE-NAME = cNewfile.
    IF FILE-INFO:FULL-PATHNAME > "" THEN
        lHTMLMapped = TRUE.
    ELSE
        lHTMLMapped = FALSE.            

    IF checksyntax OR NOT lHTMLmapped THEN
    DO:
        idx = R-INDEX(SourceFileFull, ".").    
        ASSIGN cNewfile = SUBSTRING(SourceFileFull, 1, idx) + "w".    	    
    END.

    IF NOT lHTMLMapped THEN
    DO:        
        RUN webutil/e4gl-gen.r (INPUT SourceFileFull, 
                            INPUT-OUTPUT cSpeedFile,
                            INPUT-OUTPUT cNewfile).
    END.
    ELSE IF checksyntax THEN
    DO:
        OS-COPY VALUE(FILE-INFO:FULL-PATHNAME) VALUE(cNewfile).    
    END.       
            	                  
    ASSIGN SourceFileFull = cNewfile.
END.

RUN CreateCompileTemp.

RUN VALUE(CompileTempFile) "".

OS-DELETE VALUE(CompileTempFile).

RUN parseErrors.

/* cleanup */
IF CheckSyntax OR ((SourceFile MATCHES "*~~.html" OR SourceFile MATCHES "*~~.htm") AND NOT lHTMLMapped) THEN DO:
    OS-DELETE VALUE(SourceFileFull).
END.

IF cRet = "" THEN cRet = "OK" + {&COMMAND_SEPARATOR} + COMPILER:CLASS-TYPE.


RETURN cRet.

/*
  this section initializes all the variables needed by the compile statement generator
  it is the responsability of this section to make sure all paths use forward-slashes ("/")

*/
PROCEDURE separateOptions.

	DEFINE VARIABLE i               AS INTEGER    NO-UNDO.
	
	DEFINE VARIABLE OptionCount     AS INTEGER    NO-UNDO.
	DEFINE VARIABLE cOption          AS CHARACTER  NO-UNDO.
	DEFINE VARIABLE optName         AS CHARACTER  NO-UNDO.
	DEFINE VARIABLE optValue        AS CHARACTER  NO-UNDO.
	
	OptionCount = NUM-ENTRIES(cPrm, {&COMMAND_SEPARATOR}).
	
	DO i = 1 to OptionCount:
		cOption = ENTRY(i, cPrm, {&COMMAND_SEPARATOR}).
		OptName = ENTRY(1, cOption, {&OPTION_SEPARATOR}).
		optValue = ENTRY(2, cOption, {&OPTION_SEPARATOR}).
		CASE optName:
			WHEN "LITERAL" THEN
				LiteralOptions = OptValue.
			WHEN "FILE" THEN
				SourceFile = OptValue.
			WHEN "XREF" THEN
				IF OptValue = "TRUE" THEN
					SaveXRef = true.
			WHEN "XCODEKEY" THEN
				XCodeKey = optValue.
			WHEN "SAVEINTO" THEN DO:
				IF OptValue <> "" THEN DO:
					RCodePath = optValue.
				END.
			END.
			WHEN "SAVE" THEN DO:
			  IF optValue = "true" THEN
			  	SaveRCode = true.			
			END.
			WHEN "CHECKSYNTAX" THEN DO:
				checksyntax = true.
			END.
			WHEN "TEMPFILE" THEN DO:
				SyntaxFile = optValue.
				SyntaxFile = REPLACE(SyntaxFile, "~\", "/").				
			END.
            WHEN "DEBUGLIST" THEN DO:
                DebugListing = TRUE.
                DebugListFile = optValue.
            END.
            WHEN "CODELISTING" THEN DO:
                CodeListing = TRUE.
                CodeListFile = optValue.
            END.
            WHEN "XREFFILE" THEN DO:
            	XRefFile = optValue.
            END.
            WHEN "CODELISTINGFILE" THEN DO:
            	CodeListFile = optValue.
            END.
            WHEN "DEBUGLIST" THEN DO:
            	DebugListFile = optValue.
            END.
            WHEN "XREFXML" THEN DO:
            	XRefXmlDirectory = optValue.
			END.
			
		END CASE.
	
	END.
	
END PROCEDURE.

/*
	This builds the file that gets run to compile an editor file
	This is done to trap any types of errors as well as be able
	to specify all the possible compile syntax
*/	
	
PROCEDURE CreateCompileTemp.
	OUTPUT STREAM CMPLSTREAM TO VALUE(CompileTempFile).

	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE XCodeKey         AS CHARACTER NO-UNDO." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE ErrorLogFile     AS CHARACTER NO-UNDO." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE SourceFileFull   AS CHARACTER NO-UNDO." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE CodeListFile     AS CHARACTER NO-UNDO." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE DebugListFile    AS CHARACTER NO-UNDO." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "DEFINE SHARED VARIABLE RCodePath        AS CHARACTER NO-UNDO." SKIP.

	PUT STREAM CMPLSTREAM UNFORMATTED SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "OUTPUT TO VALUE(ErrorLogFile) UNBUFFERED." SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED SKIP.
	
	PUT STREAM CMPLSTREAM UNFORMATTED "COMPILE VALUE(SourceFileFull) ".
	IF LiteralOptions <> "" THEN
		PUT STREAM CMPLSTREAM UNFORMATTED LiteralOptions + " ".
	IF SaveRCode THEN DO:
		PUT STREAM CMPLSTREAM UNFORMATTED "SAVE ".
		IF RCodePath <> "" THEN
			PUT STREAM CMPLSTREAM UNFORMATTED "INTO VALUE(RCodePath) ".
	END.
	IF XCodeKey <> "" THEN
		PUT STREAM CMPLSTREAM UNFORMATTED "XCODE XCodeKey ".	
	IF SaveXRef THEN
		PUT STREAM CMPLSTREAM UNFORMATTED "XREF VALUE(XRefFile) ".
	IF CodeListing THEN
		PUT STREAM CMPLSTREAM UNFORMATTED "LISTING VALUE(CodeListFile) ".		
	IF DebugListing THEN
		PUT STREAM CMPLSTREAM UNFORMATTED "DEBUG-LIST VALUE(DebugListFile) ".
    IF XrefXmlDirectory <> "" THEN
        PUT STREAM CMPLSTREAM UNFORMATTED 'XREF-XML "' XrefXmlDirectory '" '.
	

	PUT STREAM CMPLSTREAM UNFORMATTED "NO-ERROR.".
	PUT STREAM CMPLSTREAM UNFORMATTED SKIP.
	PUT STREAM CMPLSTREAM UNFORMATTED "OUTPUT CLOSE.".
	OUTPUT STREAM CMPLSTREAM CLOSE.

END PROCEDURE.


/*
 This reads the error messages from the compiler handle if there are any
and then puts them along with the line number and file name where the error
occured into a format that can be read by OE Architect.  This only works with
10.1B and forward.
*/
PROCEDURE parseErrors.

	DEFINE VARIABLE i 			AS INTEGER 		NO-UNDO.
	DEFINE VARIABLE msg 		AS CHARACTER 	NO-UNDO.
	DEFINE VARIABLE msgNumber 	AS INTEGER 		NO-UNDO.
	DEFINE VARIABLE msgRow		AS INTEGER 		NO-UNDO.
	DEFINE VARIABLE fileName	AS CHARACTER	NO-UNDO.
	DEFINE VARIABLE msgType		AS INTEGER      NO-UNDO.
	DEFINE VARIABLE msgColumn	AS INTEGER      NO-UNDO.
	
	ASSIGN
		cRet = "".

	IF COMPILER:NUM-MESSAGES > 0 THEN DO:
		DO i = 1 to COMPILER:NUM-MESSAGES:
			ASSIGN 
				msg = COMPILER:GET-MESSAGE(i)
				msgNumber = COMPILER:GET-NUMBER(i)
				msgRow = COMPILER:GET-ROW(i)
				msgColumn = COMPILER:GET-COLUMN(i)
				fileName   = COMPILER:GET-FILE-NAME(i)
				msgType = COMPILER:GET-MESSAGE-TYPE(i).
			IF msgRow = ? THEN
				msgRow = 0.
			IF msgNumber = ? THEN
				msgNumber = 0.
			IF fileName = ? THEN
				fileName = SourceFile.
			IF msgColumn = ? THEN
				msgColumn = 0.
			cRet = cRet + fileName + {&OPTION_SEPARATOR} + STRING(msgRow) + {&OPTION_SEPARATOR} + STRING(msgColumn) + {&OPTION_SEPARATOR}+ msg + {&OPTION_SEPARATOR} + STRING(msgNumber) + {&OPTION_SEPARATOR} + STRING(msgType) + {&COMMAND_SEPARATOR}.
		END.
	END.
	ELSE DO:
		/* this is here due to a bug in the compiler that sets compiler:num-messages to 0 when there are &message statements in a parent class */
		do i = 1 to error-status:num-messages:
			msgRow = compiler:error-row.
			msgColumn = compiler:error-column.
			if msgRow = ? then
				msgRow = 0.
			if msgColumn = ? then
				msgColumn = 0.
			msgNumber = error-status:get-number(i).
			msg = ERROR-STATUS:GET-MESSAGE(i).
			
			cRet = cRet + SourceFile + {&OPTION_SEPARATOR} + string(msgRow) + {&OPTION_SEPARATOR} + STRING(msgColumn) + {&OPTION_SEPARATOR} + msg + {&OPTION_SEPARATOR} + string(msgNumber) + {&OPTION_SEPARATOR} + STRING(msgType) + {&COMMAND_SEPARATOR}.
		end.
	END.

END PROCEDURE.

