/* ***********************************************************/
/* Copyright (c) 2004-2005 by Progress Software Corporation  */
/*                                                           */
/* All rights reserved.  No part of this program or document */
/* may be  reproduced in  any form  or by  any means without */
/* permission in writing from Progress Software Corporation. */
/*************************************************************/
   
DEFINE VARIABLE hSocket AS HANDLE NO-UNDO.
DEFINE VARIABLE aOk     AS LOGICAL NO-UNDO.

DEFINE NEW SHARED VARIABLE cPDIR    			AS CHARACTER .
DEFINE NEW SHARED VARIABLE cRoot    			AS CHARACTER .
DEFINE NEW SHARED VARIABLE cLinkedResources  	AS CHARACTER.
DEFINE VARIABLE appbuilderRunning AS LOGICAL	 NO-UNDO.

DEFINE VARIABLE cWork      AS CHARACTER  NO-UNDO.
DEFINE VARIABLE cWorkspace AS CHARACTER  NO-UNDO.
DEFINE VARIABLE hLogger    AS HANDLE     NO-UNDO.

DEFINE NEW GLOBAL SHARED VARIABLE OEIDE_ABSecEd    AS HANDLE NO-UNDO.
DEFINE NEW GLOBAL SHARED VARIABLE OEIDE_Parameters AS CHARACTER NO-UNDO.
DEFINE NEW GLOBAL SHARED VARIABLE OEIDE_ABEmbedded AS LOGICAL NO-UNDO INITIAL TRUE.

DEFINE VARIABLE OEIDE_DynamicsEnabled AS LOGICAL NO-UNDO INITIAL FALSE.

/* declaration for protocol */
&SCOPED-DEFINE MSG_HEADER_SIZE 6
&SCOPED-DEFINE PACKET_HEADER_SIZE 5
&SCOPED-DEFINE MSG_HEADER_BEGIN 254
&SCOPED-DEFINE MSG_HEADER_END   253
&SCOPED-DEFINE PACKET_BEGIN     252
&SCOPED-DEFINE PACKET_END       251
&SCOPED-DEFINE MSG_END          250

/* declarations for command format */
&SCOPED-DEFINE COMMAND_PROGRAM 1
&SCOPED-DEFINE COMMAND_PARAMETERS 2

/* 
* this is declaration for logging mechanism 
*/
FUNCTION log RETURNS CHARACTER 
	( INPUT msg AS CHARACTER) FORWARD.

&global-define DEFAULT-SERVER-PORT 3333
&global-define MAX-CONNECTION-FAILURES 3
&global-define COMMAND_SEPARATOR chr(3)
&global-define OPTION_SEPARATOR chr(4)

&IF '{&WINDOW-SYSTEM}' = 'GUI':U &THEN
  {src/adm2/globals.i}
&ENDIF

/******* Initialization ****/
cWorkspace = OS-GETENV("ECLIPSE_PROJECT"). 
IF cWorkspace = ? THEN
    cWorkspace = "unknown".
ASSIGN 
  cPDir = REPLACE(PROGRAM-NAME(1),"_server.p","")
  cRoot = OS-GETENV("ECLIPSE_ROOT")
  cRoot = REPLACE(cRoot, "~\", "/") 
  cWork = OS-GETENV("ECLIPSE_WORK")
  cLinkedResources = cWork + "/" + cWorkspace
  .

/* this will hide any session resets */
&IF '{&WINDOW-SYSTEM}' = 'GUI':U &THEN
	CURRENT-WINDOW:VISIBLE = FALSE.
	SESSION:SYSTEM-ALERT-BOXES = TRUE.
&ENDIF

CREATE SOCKET hSocket.
	

RUN ConnectToServer NO-ERROR.


IF NOT aOK OR ERROR-STATUS:ERROR or NOT hSocket:connected() THEN do:
	log( "Error Status: " + STRING(ERROR-STATUS:ERROR)).
	log( "Return Value: " + RETURN-VALUE ).
	log( "Error Message: " + ERROR-STATUS:GET-MESSAGE(1)).
	log( "aOk variable: " + STRING(aOk)).
	log( "Socket connected: " + STRING(hSocket:CONNECTED())).
	RUN QUIT ("").
	QUIT.
END.


/* now that we have connected, wait forever listening for
   data from the server
   When we receive data, pass it to the read handler and then
   start looping again. 
*/

DONTQUIT:
DO ON ERROR     UNDO, RETRY DONTQUIT
ON STOP      UNDO, RETRY DONTQUIT
ON ENDKEY    UNDO, RETRY DONTQUIT
ON QUIT      UNDO, RETRY DONTQUIT:
	WAIT-FOR "CLOSE" OF THIS-PROCEDURE.
END.

QUIT.

/* try to connect to the eclipse project running
   in the workspace 
*/

PROCEDURE ConnectToServer.
  DEFINE VARIABLE cPort AS CHARACTER NO-UNDO.

  cPort = OS-GETENV("OEA_PORT").

  if cPort = "" OR cPort = ? THEN cPort = "{&DEFAULT-SERVER-PORT}".  /* Default port */

  log( "Connecting to eclipse project").
  aOk = hSocket:CONNECT("-S " + cPort) NO-ERROR.
  log( ERROR-STATUS:GET-MESSAGE(1)).
  
  if NOT aOK THEN
    RETURN ERROR "Connection to eclipse project failed on port " + cPort.
  ELSE DO:
    RUN SendConnectionGreeting NO-ERROR.
    IF ERROR-STATUS:ERROR = TRUE THEN DO:
    	RETURN ERROR RETURN-VALUE.
    END.
  END.

END PROCEDURE.

/*  Send the workspace name back to eclipse so that eclipse knows what Progress
 *  session just connected.
*/
PROCEDURE SendConnectionGreeting:
  DEFINE VARIABLE greeting AS CHARACTER NO-UNDO.
  
  hSocket:SET-READ-RESPONSE-PROCEDURE("ReceiveCommand", THIS-PROCEDURE).   
  IF cWorkspace = "" OR cWorkspace = ? THEN DO:
  	log( "Workspace name not passed in environment").
    aOk = FALSE.
    RETURN ERROR "Workspace name not passed in environment".
  END.
  greeting = "CLIENT:" + cworkspace.
  log( "Connected").
  
  run WriteToSocket(0, greeting) NO-ERROR.
  IF ERROR-STATUS:ERROR = TRUE THEN DO:
	RETURN ERROR RETURN-VALUE.
  END.
END.

/** handles writing of response data back to the eclipse session
 *
 * Message format:
   - MSG_HEADER_BEGIN    - 1 byte      - CHR(254)
   - requestId           - 4 bytes
   - MSG_HEADER_END      - 1 byte      - CHR(253)
   - PACKET_BEGIN        - 1 byte      - CHR(252)
   - packetLength        - 4 bytes
   - data                - priceless
   - PACKET_END          - 1 byte      - CHR(251)
   - MSG_END             - 1 byte      - CHR(250)
*/


PROCEDURE WriteToSocket:

  DEFINE INPUT PARAMETER requestId AS INTEGER   NO-UNDO.
  DEFINE INPUT PARAMETER packet    AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE messageHeader    AS MEMPTR    NO-UNDO.
  DEFINE VARIABLE packetBuffer     AS MEMPTR    NO-UNDO.
  DEFINE VARIABLE messageTrailer   AS MEMPTR    NO-UNDO.
  
  DEFINE VARIABLE packetLength     AS INTEGER   NO-UNDO.
  DEFINE VARIABLE ok			   AS LOGICAL   NO-UNDO.
  DEFINE VARIABLE msg              AS CHARACTER NO-UNDO.
  
  IF packet = ? THEN
    packet = "?".
    
  ok = VALID-HANDLE(hSocket).
  
  IF ok = TRUE  THEN DO:
    ok = hSocket:CONNECTED().
    
    IF ok = TRUE THEN DO:
      	
      SET-SIZE(messageHeader) = {&MSG_HEADER_SIZE}.
      PUT-BYTE(messageHeader, 1) = {&MSG_HEADER_BEGIN}.
      PUT-LONG(messageHeader, 2) = requestId.
      PUT-BYTE(messageHeader, 6) = {&MSG_HEADER_END}.
      
	  ok = hSocket:WRITE (messageHeader,1,{&MSG_HEADER_SIZE}) NO-ERROR.      
      IF ok = TRUE THEN DO:
	      packetLength = LENGTH(packet).
	      
	  	  SET-SIZE(packetBuffer) = packetLength + {&PACKET_HEADER_SIZE} + 1.
	  	  PUT-BYTE(packetBuffer, 1) = {&PACKET_BEGIN}.
	  	  PUT-LONG(packetBuffer, 2) = packetLength.  	  
		  PUT-STRING(packetBuffer, 6, LENGTH(packet)) = packet.
		  PUT-BYTE(packetBuffer, {&PACKET_HEADER_SIZE} + packetLength + 1) = {&PACKET_END}.
		  	  	
		  ok = hSocket:WRITE (packetBuffer,1, {&PACKET_HEADER_SIZE} + packetLength + 1) NO-ERROR.
		  
		  IF ok = TRUE THEN DO:
			  SET-SIZE(messageTrailer) = 1.	  
			  PUT-BYTE(messageTrailer, 1) = {&MSG_END}.
		      ok = hSocket:WRITE (messageTrailer,1, 1) NO-ERROR.
		  END.
	
      END.
      
    END.
    
  END.

  SET-SIZE(messageHeader)  = 0.
  SET-SIZE(packetBuffer)   = 0.    
  SET-SIZE(messageTrailer) = 0.
	
  IF OK <> TRUE THEN DO:
    msg = ERROR-STATUS:GET-MESSAGE(1).
    RUN QUIT(msg).
    RETURN ERROR msg.
  END.
	
END.

/*
This handles incoming commands from the Eclipse JVM.
This uses the same format as outgoing messages
*/
PROCEDURE ReceiveCommand:
  
  /* Read procedure for socket */
  DEFINE VARIABLE requestId      AS INTEGER   NO-UNDO.
  DEFINE VARIABLE messageSize    AS INTEGER   NO-UNDO.
  DEFINE VARIABLE oneByte        AS MEMPTR    NO-UNDO.
  DEFINE VARIABLE fourBytes      AS MEMPTR    NO-UNDO.
  DEFINE VARIABLE theData        AS MEMPTR    NO-UNDO.
  DEFINE VARIABLE cError         AS CHARACTER NO-UNDO.
  DEFINE VARIABLE ok             AS LOGICAL   NO-UNDO.

  IF NOT SELF:CONNECTED() THEN
  DO:
	RUN QUIT ("Lost connection").
	RETURN ERROR "Socket disconnected".
  END.

  SET-SIZE(oneByte) = 1.
  SET-SIZE(fourBytes) = 4.
  SELF:READ(oneByte, 1, 1).
  IF GET-BYTE(oneByte, 1) = {&MSG_HEADER_BEGIN} THEN DO:
    SELF:READ(fourBytes, 1, 4).
    requestId = GET-LONG(fourBytes, 1).
    IF requestId > -1 THEN DO:
      SELF:READ(oneByte, 1, 1).
      IF GET-BYTE(oneByte, 1) = {&MSG_HEADER_END} THEN DO:
        SELF:READ(oneByte, 1, 1).
        IF GET-BYTE(oneByte, 1) = {&PACKET_BEGIN} THEN DO:
          SELF:READ(fourBytes, 1, 4).
          messageSize = GET-LONG(fourBytes, 1).
          IF messageSize > -1 THEN DO:
            SET-SIZE(theData) = messageSize.
            ok = SELF:READ(theData, 1, messageSize, READ-EXACT-NUM).
            IF ok = TRUE THEN DO:
            SELF:READ(oneByte, 1, 1).
            IF GET-BYTE(oneByte, 1) = {&PACKET_END} THEN DO:
              SELF:READ(oneByte, 1, 1).
              IF GET-BYTE(oneByte, 1) = {&MSG_END} THEN DO:
              END.
              ELSE
                cError = "Message End Marker".              
            END.
            ELSE
              cError = "Packet End Marker".
            END.
            ELSE
               cError = "Failed to read all of message data".
          END.
          ELSE
            cError = "Packet Size".
        END.
        ELSE
          cError = "Packet Begin".
      END.
      ELSE
        cError = "Message Header End".
    END.
    ELSE
      cError = "Request Id".
  END.
  ELSE
    cError = "Message Header Begin".
      
  DEFINE VARIABLE cCmd       AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cPrm       AS CHARACTER NO-UNDO.  
  
  IF cError = "" THEN DO:
    RUN extractCommand(INPUT theData, OUTPUT cCmd, OUTPUT cPrm).
    IF (ERROR-STATUS:ERROR) THEN 
      cError = "ERROR:" + ERROR-STATUS:GET-MESSAGE(1).
  END.      
      
  SET-SIZE(fourBytes) = 0.
  SET-SIZE(oneByte) = 0.
  SET-SIZE(theData) = 0.  

  IF cError <> "" THEN DO:
    RUN QUIT("Error: " + cError).
    RETURN ERROR cError.
  END.
  ELSE DO:
    RUN executeCmd( cCmd, cPrm, requestID) NO-ERROR.
    IF ERROR-STATUS:ERROR = TRUE THEN DO:
    	RETURN ERROR RETURN-VALUE.
    END.
  END.  
  
END PROCEDURE.

/* If a command was received from eclipse then process it here and
*  call the appropriate procedure.
*  Commands are sent in the format of type, length, command string 
*  followed by type, length, parameter string
*/

PROCEDURE executeCmd:
  DEFINE INPUT PARAMETER pcCmd        AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pcPrm        AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER requestId    AS INTEGER   NO-UNDO.

  DEFINE VARIABLE cRet AS CHARACTER NO-UNDO.

  ERROR-STATUS:ERROR = FALSE NO-ERROR.
  DO ON ERROR UNDO, LEAVE
     ON STOP UNDO, LEAVE:
     	 
	log( "Request  " + STRING(requestId) + ": " + pcCmd + "(" + pcPrm + ")").
          
	IF NOT CAN-DO(THIS-PROCEDURE:INTERNAL-ENTRIES, pcCmd) THEN DO:
		FILE-INFO:FILE-NAME = pcCmd.
		IF FILE-INFO:FULL-PATHNAME = ? 
			OR (INDEX(FILE-INFO:FILE-TYPE, "F") = 0 AND INDEX(FILE-INFO:FILE-TYPE, "M") = 0) THEN DO:
			pcCmd = SEARCH(cPDir + "_ide" + pcCmd + ".p").
		END.
	END.
             
	RUN ClearReturnValue NO-ERROR.
	IF pcCmd > "" THEN DO: 
		DONTQUIT:
		DO ON ERROR     UNDO, LEAVE DONTQUIT
			ON STOP     UNDO, LEAVE DONTQUIT
			ON ENDKEY   UNDO, LEAVE DONTQUIT
			ON QUIT     UNDO, LEAVE DONTQUIT:
			
			RUN VALUE(pcCmd)(pcPrm). 
		END.
		cRet = RETURN-VALUE.
	END.
	ELSE DO:
		cRet = "ERROR:FileNotFound".
	END.
  END.
  
  IF cRet = "" AND (ERROR-STATUS:ERROR OR ERROR-STATUS:GET-MESSAGE(1) > "") THEN 
    cRet = "ERROR:" + ERROR-STATUS:GET-MESSAGE(1).
 
  log( "Response " + STRING(requestId) + ": " + cRet).
  RUN WriteToSocket(requestId, cRet) NO-ERROR.
  IF ERROR-STATUS:ERROR OR RETURN-VALUE <> "" THEN
    RETURN ERROR RETURN-VALUE.
    
END PROCEDURE.

PROCEDURE extractCommand:
  	DEFINE INPUT PARAMETER requestData  AS MEMPTR    NO-UNDO.
	DEFINE OUTPUT PARAMETER pcCmd       AS CHARACTER NO-UNDO.
	DEFINE OUTPUT PARAMETER pcPrm       AS CHARACTER NO-UNDO.
	
     define variable cmdType as integer no-undo.
     define variable cmdSize as integer no-undo.
     define variable prmType as integer no-undo.
     define variable prmSize as integer no-undo.
     
     define variable iPos as integer no-undo init 1.

  ERROR-STATUS:ERROR = FALSE NO-ERROR.
  DO ON ERROR UNDO, LEAVE
     ON STOP UNDO, LEAVE:     	
	 cmdType = GET-LONG(requestData, 1).
	 iPos = iPos + 4.
	 cmdSize = GET-LONG(requestData, iPos).
	 iPos = iPos + 4.
	 IF cmdSize > 0 THEN DO:
		 pcCmd = GET-STRING(requestData, iPos, cmdSize).
		 iPos = iPos + cmdSize.
     END.
	 prmType = GET-LONG(requestData, iPos).
	 iPos = iPos + 4.
	 prmSize = GET-LONG(requestData, iPos).
	 iPos = iPos + 4.
	 IF prmSize > 0 THEN DO:
		 pcPrm = GET-STRING(requestData, iPos, prmSize).
		 iPos = iPos + prmSize.
	 END.		
  END.	 
END.

/*
*  This is here since it is the only way to clear the return value
*  and the error status flag.
*/
PROCEDURE ClearReturnValue:
	RETURN "".
END.

/*
*  This terminates the infinite wait-for "close"
*  and cleans up
*/
PROCEDURE QUIT:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	IF appbuilderRunning THEN DO:
		RUN closeAppBuilder ("").
		RUN removeLinkedFiles.
	END.
	
	IF VALID-HANDLE(hSocket) THEN DO:
		IF hSocket:CONNECTED() THEN DO:
			hSocket:DISCONNECT().
		END.	
	END.
	hSocket = ?.
	IF cPrm <> ? AND LENGTH(cPrm) > 0 THEN
		log(cPrm).
	log("Quitting.").
	RUN CloseLogFile (INPUT "").
  
	APPLY "CLOSE" TO THIS-PROCEDURE.
	RETURN "TERMINATED.".
	
END PROCEDURE.

/* retrieve the propath
*/
PROCEDURE PROPATH:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  IF cPrm <> "" THEN PROPATH = cPrm.
  RETURN REPLACE(PROPATH,",",";").
END PROCEDURE.

PROCEDURE isAppBuilderRunning:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  RETURN IF appbuilderRunning THEN "TRUE" ELSE "FALSE".
END.

PROCEDURE getWindowThreadId:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  DEFINE VARIABLE result      AS INTEGER   NO-UNDO.
  
  RUN GetCurrentThreadId(OUTPUT result).
  RETURN STRING(result).  
END.


/* Return the proversion for an old project
*/
PROCEDURE proversion:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  RETURN PROVERSION.
END PROCEDURE.

/* run a particular procedure persistently
*/
PROCEDURE launch:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	DEFINE VARIABLE h as HANDLE.
	log( "Running " + cPrm + " persistently").
	RUN adecomm/_runcode.p ( INPUT  cPrm,
                           INPUT  "_PERSISTENT" ,
                           INPUT  ? ,
                           OUTPUT h ) .
	IF VALID-HANDLE(h) THEN DO:
		RETURN "OK".
	END.
	ELSE DO:
		RETURN "ERROR:" + cPrm.
	END.

END PROCEDURE.

PROCEDURE launch_with_param:
  DEFINE INPUT PARAMETER cCmd AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE h as HANDLE.
  DEFINE VARIABLE cPrm AS CHARACTER   NO-UNDO.
  
  ASSIGN
    cCmd = cCmd + " "
    cPrm = TRIM(SUBSTRING(cCmd,INDEX(cCmd,' ') + 1))
    cCmd = TRIM(ENTRY(1,cCmd,' ')).
    
	    
	RUN VALUE(cCmd) PERSISTENT SET h (INPUT cPrm) NO-ERROR.
	IF ERROR-STATUS:ERROR OR ERROR-STATUS:GET-MESSAGE(1) > "" THEN DO:
		log( "Error: " + cPrm + " " + ERROR-STATUS:GET-MESSAGE(1)).	
  		MESSAGE ERROR-STATUS:GET-MESSAGE(1)
  			VIEW-AS ALERT-BOX ERROR.
  		RETURN "ERROR:" + cPrm.
	END.
	ELSE DO:
		RETURN "OK".
	END.
END PROCEDURE.

PROCEDURE runproc:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  
	RUN VALUE(cPrm) NO-ERROR.
	IF ERROR-STATUS:ERROR OR ERROR-STATUS:GET-MESSAGE(1) > "" THEN DO:
		log( "Error: " + cPrm + " " + ERROR-STATUS:GET-MESSAGE(1)).	
  		MESSAGE ERROR-STATUS:GET-MESSAGE(1)
  			VIEW-AS ALERT-BOX ERROR.
  		RETURN "ERROR:" + cPrm.
	END.
	ELSE DO:
		RETURN "OK".
	END.
END PROCEDURE.


PROCEDURE runproc_with_param:
	DEFINE INPUT PARAMETER cCmd AS CHARACTER NO-UNDO.
  
	DEFINE VARIABLE h as HANDLE.
	DEFINE VARIABLE cPrm AS CHARACTER   NO-UNDO.
  
	ASSIGN
		cCmd = cCmd + " "
    	cPrm = TRIM(SUBSTRING(cCmd,INDEX(cCmd,' ') + 1))
    	cCmd = TRIM(ENTRY(1,cCmd,' ')).
 
	RUN VALUE(cCmd) (INPUT cPrm) NO-ERROR.
	IF ERROR-STATUS:ERROR OR ERROR-STATUS:GET-MESSAGE(1) > "" THEN DO:
		log( "Error: " + cPrm + " " + ERROR-STATUS:GET-MESSAGE(1)).	
  		MESSAGE ERROR-STATUS:GET-MESSAGE(1)
  			VIEW-AS ALERT-BOX ERROR.
  		RETURN "ERROR:" + cPrm.
	END.
	ELSE DO:
		RETURN "OK".
	END.
END PROCEDURE.

/* run a program non-persist
*/
PROCEDURE rungui:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  DEFINE VARIABLE h as HANDLE.
  log( "Running " + cPrm).
  RUN adecomm/_runcode.p ( INPUT  cPrm,
                           INPUT  "_PAUSE,_TOFRONT" ,
                           INPUT  ? ,
                           OUTPUT h ) .
  log( "Completed " + cPrm).
  RETURN "OK:" + cPrm .
END PROCEDURE.

/* run a program and set a breakpoint at the first line
*/ 
PROCEDURE rundebug:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  DEFINE VARIABLE h as HANDLE.
  log( "Debugging " + cPrm).
  
  DEBUGGER:INITIATE().
  DEBUGGER:SET-BREAK(cPrm).
  RUN adecomm/_runcode.p ( INPUT  cPrm,
                           INPUT  "_PAUSE" ,
                           INPUT  ? ,
                           OUTPUT h ) .
  DEBUGGER:CLEAR().  
  DEBUGGER:VISIBLE = FALSE.

  log( "Completed Debugging" + cPrm).
  RETURN "OK:" + cPrm .
END PROCEDURE.

/* perform a Progress file search on the specified file
*/
PROCEDURE SEARCH:
  DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cProject AS CHARACTER.
  DEFINE VARIABLE cFilename AS CHARACTER NO-UNDO.
  ASSIGN 
    cFilename = SEARCH(cPrm).
  IF cFilename = ? THEN RETURN "ERROR:FileNotFound".
  ASSIGN
    FILE-INFO:FILE-NAME = cFileName
    cFileName = FILE-INFO:FULL-PATHNAME
    cFilename = REPLACE(cFilename,"~\","/")
    cFilename = REPLACE(cFilename,cRoot,"").
  RETURN cFilename.  
END PROCEDURE.

/* set the AppBuilder to be used in embedded mode
*/
PROCEDURE setabembedded:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  OEIDE_ABEmbedded = (cprm = "true").
  RETURN "OK".
END PROCEDURE.

/* Set Dynamics enabled flag
*/
PROCEDURE setdynamicsenabled:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  OEIDE_DynamicsEnabled = (cprm = "true").
  RETURN "OK".
END PROCEDURE.

/* Run the appbuilder
*/
PROCEDURE runAppBuilder:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	DEFINE VARIABLE h AS HANDLE     NO-UNDO.
  
	RUN isAppBuilderRunning (INPUT "").
	IF (RETURN-VALUE = "TRUE") THEN DO:
		RETURN "AppBuilder Already Running".
	END.
  
	appbuilderRunning = TRUE.
		
	RUN ClearReturnValue.
	DO	ON ERROR     UNDO, LEAVE
		ON STOP      UNDO, LEAVE
		ON QUIT      UNDO, LEAVE:
	IF OEIDE_DynamicsEnabled THEN
		RUN icfstart.p.
	ELSE
		RUN _ab.p.				
	END.		
	IF VALID-HANDLE(OEIDE_ABSecEd) THEN DO:
		DELETE OBJECT OEIDE_ABSecEd NO-ERROR.	  
	END.
	appbuilderRunning = FALSE.
	
	IF hSocket = ? THEN
		APPLY "CLOSE" TO THIS-PROCEDURE.	
	
	RETURN "OK".
END PROCEDURE.

PROCEDURE closeAppBuilder:
    DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	DEFINE VARIABLE h AS HANDLE NO-UNDO.
	
	IF NOT appbuilderRunning THEN RETURN.     
	h = SESSION:FIRST-CHILD.
	DO WHILE VALID-HANDLE(h):
		IF h:NAME = ? AND h:TITLE = "AppBuilder" THEN
	    	DO:
	        	APPLY "U9":U TO h.
				LEAVE.
		END.
		h = h:NEXT-SIBLING.
	END.
	IF VALID-HANDLE(OEIDE_ABSecEd) THEN
		DELETE OBJECT OEIDE_ABSecEd NO-ERROR.	
	appbuilderRunning = FALSE.  
	RETURN "OK".
END PROCEDURE.

PROCEDURE removeLinkedFiles:
	DEFINE VARIABLE h AS HANDLE NO-UNDO.
	     
	h = SESSION:FIRST-PROCEDURE.
	DO WHILE VALID-HANDLE(h)
         AND h:FILE-NAME <> "adecomm/oeideservice.p":U:
		h = h:NEXT-SIBLING.
	END.
	IF VALID-HANDLE(h) THEN
	    RUN removeLinkedFiles IN h NO-ERROR.	    
END PROCEDURE.

PROCEDURE syncFromIDE:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	  RUN syncFromIDE IN OEIDE_ABSecEd (cprm) NO-ERROR.
  RETURN "Ok:syncFromIDE".	  
END.

PROCEDURE syncFromAB:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  
  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	  RUN syncFromAB IN OEIDE_ABSecEd (cprm) NO-ERROR.
  RETURN "Ok:syncFromAB".	  	  
END.

PROCEDURE openABFile:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.

  IF appbuilderRunning THEN
  DO:
	  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	      RUN openABFile IN OEIDE_ABSecEd (cprm) NO-ERROR.
  END.	      
  ELSE
  DO:
	  OEIDE_Parameters = cprm.
	  RUN runAppBuilder ("").
  END.	
  RETURN "Ok:openABFile".  
END.

PROCEDURE sendABFileEvent:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  
  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	  RUN sendABFileEvent IN OEIDE_ABSecEd (cprm) NO-ERROR.
  RETURN "Ok:sendABFileEvent".
END.

PROCEDURE closeABFile:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  
  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	  RUN closeABFile IN OEIDE_ABSecEd (cprm) NO-ERROR.
  RETURN "Ok:closeABFile".	  
END.

PROCEDURE renameABFile:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  
  IF VALID-HANDLE(OEIDE_ABSecEd) THEN
	  RUN renameABFile IN OEIDE_ABSecEd (cprm) NO-ERROR.  
  RETURN "Ok:renameABFile".	  	  
END.

/* generate a debug list file
*/
PROCEDURE gendebug:
  DEFINE INPUT  PARAMETER cprm AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE oldPropath AS CHARACTER  NO-UNDO.
  PROPATH = cLinkedResources + "/temp," + PROPATH.
  COMPILE VALUE(cLinkedResources + "/temp/0.t") DEBUG-LIST VALUE(cLinkedResources + "/temp/debug.p").
END PROCEDURE.


/* retrieve the schema for any connected database
*/
PROCEDURE getDBSchemas:
  DEFINE INPUT  PARAMETER cPrm AS CHARACTER  NO-UNDO.

  log( "CREATING SCHEMAS").
  RUN VALUE(cPDir + "_dblist.p") (cWork).  /* Pass table name if given */	
  RETURN RETURN-VALUE.
END PROCEDURE.


PROCEDURE generateDBSchemas:
/*------------------------------------------------------------------
 This will get generate the schema cache for the specified schemas

 Input Parameters:
   p_DBNameList  A comma delimited list of the name of the databases
---------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER  p_DBNameList  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE cDB    		AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i1   	 		AS INTEGER  NO-UNDO.
  DEFINE VARIABLE j   	 		AS INTEGER  NO-UNDO.
  DEFINE VARIABLE guidEntry 	AS CHARACTER NO-UNDO.
  DEFINE VARIABLE guidList  	AS CHARACTER NO-UNDO.
  DEFINE VARIABLE guidDoneList  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE timeStampList  AS CHARACTER NO-UNDO.

  log( "CREATING SCHEMAS FOR " + p_DBNameList).
  RUN getDBLocationID(p_DBNameList).
  guidList = RETURN-VALUE.
  
  RUN getDBTimeStamp(p_DBNameList).
  timeStampList = RETURN-VALUE.
  DO i1 = 1 to NUM-ENTRIES(p_DBNameList):
    cDB = ENTRY(i1,p_DBNameList).
    guidEntry = ENTRY(i1,guidList).    
    guidEntry = REPLACE(guidEntry,"~\","-").
    guidEntry = REPLACE(guidEntry,"/","-").
    /* Skip if already created */
    IF LOOKUP(guidEntry,guidDoneList) > 0 THEN
       NEXT.
    OUTPUT TO VALUE(cWork + (guidEntry) + ".txt").
    /* Write out header */
    PUT UNFORMATTED  "PSC-SchemaCache|" +  ENTRY(i1,timeStampList)   SKIP.
    RUN VALUE(cPDir + "_idecreatealias.p") (cDB, "DICTDB").
	RUN VALUE(cPDir + "_exporttables.p").
    OUTPUT CLOSE.
    guidDoneList = guidDoneList + (IF guidDoneList = "" THEN "" ELSE ",") 
                                + guidEntry.
  END.
  
END PROCEDURE.


PROCEDURE getDBLocationID:
/*------------------------------------------------------------------
 This will get the unique ID of the database (guiID). If the database
 does not contain the guid, the logical database name will be returned

Input Parameters:
   p_DBName  - This is the name of the database you want to find.

Returns - A delimited list of _DB._DB-guid, DBNAME if the db was not found.
---------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER p_DBNameList    as char     NO-UNDO.

  DEFINE VARIABLE  dbEntry          AS CHARACTER NO-UNDO.
  DEFINE VARIABLE dbLocationID   	AS CHARACTER NO-UNDO.
  DEFINE VARIABLE ret-value   		AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i                 AS INTEGER NO-UNDO.
  DEFINE VARIABLE v_DBCount         AS INTEGER NO-UNDO.
 
  DO i = 1 TO NUM-ENTRIES(p_DBNameList):
    dbEntry  = ENTRY(i,p_DBNameList).
    
    /* if this is a schema holder, then find it in the right progress db */
	IF NOT CONNECTED(dbEntry) OR DBTYPE(dbEntry) NE "Progress" THEN
	DO:
	  DB-LOOP:
	  DO v_DBCount = 1 to num-dbs:
	    IF DBTYPE(v_DBCount) = "PROGRESS" THEN
	    DO:
	      create alias dictdb for database VALUE(LDBNAME(v_DBCount)).
	      RUN VALUE(cPDir + "_hasdbguid.p")  NO-ERROR.
	      IF RETURN-VALUE = "TRUE" THEN DO:
	      	RUN VALUE(cPDir + "_finddbguid.p")  NO-ERROR.
	      	ret-value = IF RETURN-VALUE= ? THEN dbEntry ELSE RETURN-VALUE.
	      END.	
	      ELSE
	        ret-value = dbEntry.
	      
          dbLocationID = dbLocationID + (IF dbLocationID = "" THEN "" ELSE ",")
	                                + ret-value.                 
	      LEAVE DB-LOOP.                          
	    END.
	  END.
	END.
	ELSE /* just put the alias on this db */
	DO:
	  IF LDBNAME("DICTDB") <> dbEntry THEN
	    create alias DICTDB for database VALUE(dbEntry).
	  RUN VALUE(cPDir + "_hasdbguid.p")  NO-ERROR.
	  IF RETURN-VALUE = "TRUE" THEN DO:
	  	RUN VALUE(cPDir + "_finddbguid.p") NO-ERROR.
	  	ret-value = IF RETURN-VALUE= ? THEN dbEntry ELSE RETURN-VALUE.
	  END.	
	  ELSE
	    ret-value = dbEntry.
	  dbLocationID = dbLocationID + (IF dbLocationID = "" THEN "" ELSE ",")
	                             + ret-value.                        
	END.
  END.

RETURN dbLocationID .


END PROCEDURE.



PROCEDURE getDBTimeStamp:
/*------------------------------------------------------------------
 This will return the timeStamp id of the schema from the specified 
 databases. If you pass the name of a gateway database, it will find 
 the associated db.
 Input Parameters:
    p_DBNameList  - A comma delimited list of the logical names of the 
    		      databases you want to find.

 Returns: - a delimited list of the_MstrBlk._MstrBlk-timestamp
---------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER p_DBNameList  AS CHARACTER NO-UNDO.

  DEFINE VARIABLE  dbEntry           AS CHARACTER NO-UNDO.
  DEFINE VARIABLE dbTimeStampList   AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i                 AS INTEGER NO-UNDO.
  DEFINE VARIABLE v_DBCount         AS INTEGER NO-UNDO.

  
  DO i = 1 TO NUM-ENTRIES(p_DBNameList):
    dbEntry  = ENTRY(i,p_DBNameList).
    
    /* if this is a schema holder, then find it in the right progress db */
	IF NOT CONNECTED(dbEntry) OR DBTYPE(dbEntry) NE "Progress" THEN
	DO:
 	  DB-LOOP:
	  DO v_DBCount = 1 to num-dbs:
	    IF DBTYPE(v_DBCount) = "PROGRESS" THEN
	    DO:
	      create alias dictdb for database VALUE(LDBNAME(v_DBCount)).
	      RUN VALUE(cPDir + "_finddbtimestamp.p")  NO-ERROR.
	      IF RETURN-VALUE NE ? THEN
               dbTimeStampList = dbTimeStampList + (IF dbTimeStampList = "" THEN "" ELSE ",")
	                                + RETURN-VALUE.
	      ELSE
               dbTimeStampList = dbTimeStampList + (IF dbTimeStampList = "" THEN "" ELSE ",")
	                                + "?".                        
	      LEAVE DB-LOOP.                          
	   END.
	  END.
	END.
	ELSE /* just put the alias on this db */
	DO:
	  IF LDBNAME("DICTDB") <> dbEntry THEN
	    create alias DICTDB for database VALUE(dbEntry).
	  RUN VALUE(cPDir + "_finddbtimestamp.p") NO-ERROR.
      IF RETURN-VALUE NE ? THEN
	        dbTimeStampList = dbTimeStampList + (IF dbTimeStampList = "" THEN "" ELSE ",")
	                                + RETURN-VALUE.
	  ELSE
     	    dbTimeStampList = dbTimeStampList + (IF dbTimeStampList = "" THEN "" ELSE ",")
	                                + "?".                        
	END.
  END.
RETURN dbTimeStampList .

END PROCEDURE.

/* retrieve the description for a particular field or table
*/
PROCEDURE getDescription:
  DEFINE INPUT  PARAMETER cPrm AS CHARACTER  NO-UNDO.
  cPrm = cPrm + "..".
  CREATE ALIAS DICTDB FOR DATABASE VALUE(ENTRY(1,cPrm,".")) NO-ERROR.
  IF ERROR-STATUS:ERROR OR NOT CONNECTED("DICTDB") THEN
  	RETURN "OK:".
  ELSE DO:
	  RUN VALUE(cPDir + "_dbdescription.p") (INPUT cPrm) NO-ERROR.  /* Pass table name if given */
	  IF ERROR-STATUS:ERROR THEN
	  	RETURN "OK".
  END.
  RETURN RETURN-VALUE.
END PROCEDURE.

/*
  execute a webspeed web object file
*/
PROCEDURE webspeed:
  DEFINE INPUT PARAMETER cPrm as CHARACTER NO-UNDO.
  DEFINE VARIABLE WebObject as CHARACTER NO-UNDO.
  DEFINE VARIABLE DataFile as CHARACTER NO-UNDO.
  
  webobject = ENTRY(1, cPrm, "|").
  datafile  = ENTRY(2, cPrm, "|").
  
  log( "WEBSPEED:" + webobject + "=>" + datafile).
  RUN VALUE(cPDir + "_wswebspeed.p") (WebObject, DataFile).
  RETURN "OK".
END PROCEDURE.

PROCEDURE ping:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	
	return "OK".
END PROCEDURE.

PROCEDURE pingProgressSession:
	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.
	
	return "OK:pingProgressSession:" + cPrm.
END PROCEDURE.


FUNCTION log RETURNS CHARACTER 
	( INPUT msg AS CHARACTER).
	
	IF VALID-HANDLE(hLogger) THEN DO:
	
		DYNAMIC-FUNCTION("log" in hLogger, msg).
	END.
	
END FUNCTION.

/*
	start logging process and open the log file
*/
PROCEDURE OpenLogFile.

	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.

	IF VALID-HANDLE(hLogger) THEN DO:
		RUN CloseLogFile ("").
	END.
	
	RUN VALUE(cPDir + "_idelog.p") PERSISTENT SET hLogger.
	RUN SetLogFile IN hLogger (cPrm).
	RUN OpenLogFile IN hLogger.

	log( "PATH=" + OS-GETENV("PATH")).
	log( "DLC=" + OS-GETENV("DLC")).
	log( "PROPATH=" + PROPATH).
	log( "ECLIPSE_PROJECT=" + cWorkspace).
	log( "OEA_PORT=" + OS-GETENV("OEA_PORT")).
	log( "ECLIPSE_ROOT=" + OS-GETENV("ECLIPSE_ROOT")).
	

END PROCEDURE.

PROCEDURE CloseLogFile.

	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.

	IF VALID-HANDLE(hLogger) THEN DO:
	
		RUN CloseLogFile IN hLogger.
		DELETE OBJECT hLogger.
		hLogger = ?.
	END.

END PROCEDURE.

/* receives an event from the ide and republishes it 
   to any subsribed ABL programs.  Anyone is free
   to subscribe to this method.  Unlike _adeevent.p these events
   are simply notifications are no response to them is possible.  These are
   not published in a specific handle so to subscribe to them use the following
   syntax:
   
   SUBSCRIBE PROCEDURE THIS-PROCEDURE TO "OEIDE_EVENT" ANYWHERE.
   
   Parameters:
   
   eventName	This is the name of the event.  It is always passed.
   projectName	This is always passed and will always be the name of the project
   				in context.  Only events from the current project are passed so each
   				OpenEdge runtime environment will only receive events from a single
   				project.
   programName	This is only published for file operations.  This is the full
   				name file location of the program in context
   eventData	This is unused at the moment, but may be used in the future.
*/
PROCEDURE PublishIDEEvent.

	DEFINE INPUT PARAMETER cPrm AS CHARACTER NO-UNDO.

	DEFINE VARIABLE eventName   AS CHARACTER NO-UNDO.
	DEFINE VARIABLE projectName AS CHARACTER NO-UNDO.
	DEFINE VARIABLE programName AS CHARACTER NO-UNDO.
	DEFINE VARIABLE eventData   AS CHARACTER NO-UNDO.
	
	ASSIGN
		eventName   = ENTRY(1, cPrm, {&OPTION_SEPARATOR})
		projectName = ENTRY(2, cPrm, {&OPTION_SEPARATOR})
		programName = ENTRY(3, cPrm, {&OPTION_SEPARATOR})
		eventData   = ENTRY(4, cPrm, {&OPTION_SEPARATOR}).
	
	PUBLISH "OEIDE_EVENT" (INPUT eventName, INPUT projectName, INPUT programName, INPUT eventData).
		
END PROCEDURE.


&IF '{&WINDOW-SYSTEM}' = 'TTY':U &THEN
PROCEDURE GetCurrentThreadId:
  DEFINE OUTPUT PARAMETER windowID as INTEGER NO-UNDO.
  windowID = 0.
END.
&ELSE
PROCEDURE GetCurrentThreadId EXTERNAL "kernel32.dll":
    DEFINE RETURN PARAMETER result AS LONG.
END.
&ENDIF
