/* How to CREATE a NEW OE Architect Project in ABL*/
/* Annotation Examples */
@Supercorp.WorkOrder(WO = '2345', Programmer = 'paulp', Version = '2.11').
@SuperCorp.DocumentRef(DocName='C:\Work Files\Moitaka\Moitaka Goals.doc').

/* Setup Globals and Variables */
	&GLOBAL-DEFINE CRLF1 CHR(13)
    DEFINE VARIABLE ProjectName AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE cWork AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE cProject AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE myXML AS CHARACTER  NO-UNDO.

/* Get the new ProjectName */
    UPDATE ProjectName.

/* GET the CURRENT workspace/project directory and project name */
    cWork = OS-GETENV("ECLIPSE_ROOT").
    cProject = OS-GETENV("ECLIPSE_PROJECT").
    cWork = REPLACE(cWork, cProject, ProjectName).
    DISPLAY cWork FORMAT "x(40)".


/* Create the Project Directories */
    OS-CREATE-DIR VALUE(cWork).
    OS-CREATE-DIR VALUE(cWork + "src").
    OS-CREATE-DIR VALUE(cWork + "rcode").

/* Create .project file and Make it an OE Project.  Set BUILDER
   ie compiler and NATURE ie com.openedge.pdt.text.progressNature */
/* Note: You could also set Jet Template settings (if required) */
    RUN createProject.



/* Create .propath file.  Set {@WORK}, {@ROOT}\src \rcode */
    RUN createPropath.

/* Create .dbconnections file.
   Note:  The Connections must be already setup in DB Navigator.
          {@ECLIPSE_ROOT}..\.metadata\.plugins\com.openedge.pdt.text\dbconnections.xml */
@SuperCorp.DocumentRef(DocName='C:\Work Files\Moitaka\ws1\.metadata\.plugins\com.openedge.pdt.text\databaseConnection.xml').
    RUN createDBConnections.

/* Create project.xml file and load settings. Sample follows:
    <void method="put">
     <string>web_aliases</string>
     <string>webtools,@{DLC}/tty,webspeed,@{DLC}/webspeed</string>
    </void>
    <void method="put">
     <string>pvm_startup</string>
     <string>-d dmy -param abc -Mn 1024</string>
    </void>
 */
@SuperCorp.DocumentRef(DocName='C:\Work Files\Moitaka\ws1\Moitaka1\project.xml').



PROCEDURE createProject.
  OUTPUT TO VALUE(cWork + '.project').
  ASSIGN myXML =
     '<?xml version=~"1.0~" encoding=~"UTF-8~"?>' + {&CRLF1} +
     '<projectDescription> ' + {&CRLF1} +
	 '  <name>' + ProjectName + '</name>' + {&CRLF1} +
	 '  <comment></comment>' + {&CRLF1}  +
	 '  <projects>' + {&CRLF1} +
	 '  </projects>' + {&CRLF1} +
	 '  <buildSpec>' + {&CRLF1} +
	 '  	<buildCommand>' + {&CRLF1} +
	 '  		<name>org.eclipse.jdt.core.javabuilder</name>' + {&CRLF1} +
	 '  		<arguments>' + {&CRLF1} +
	 '  		</arguments>' + {&CRLF1} +
	 '  	</buildCommand>' + {&CRLF1} +
	 '  	<buildCommand>' + {&CRLF1} +
	 '  		<name>org.eclipse.emf.codegen.JETBuilder</name>' + {&CRLF1} +
	 '  		<arguments>' + {&CRLF1} +
	 '  		</arguments>' + {&CRLF1} +
	 '  	</buildCommand>' + {&CRLF1} +
	 '  	<buildCommand>' + {&CRLF1} +
	 '  		<name>com.openedge.pdt.text.progressBuilder</name>' + {&CRLF1} +
	 '  		<arguments>' + {&CRLF1} +
	 '  		</arguments>' + {&CRLF1} +
	 '  	</buildCommand>' + {&CRLF1} +
	 '  </buildSpec>' + {&CRLF1} +
	 '  <natures>' + {&CRLF1} +
	 '  	<nature>com.openedge.pdt.text.progressNature</nature>' + {&CRLF1} +
	 '  	<nature>org.eclipse.emf.codegen.jet.IJETNature</nature>' + {&CRLF1} +
	 '  	<nature>org.eclipse.jdt.core.javanature</nature>' + {&CRLF1} +
	 '  </natures>' + {&CRLF1} +
     '</projectDescription>' + {&CRLF1}
     .

  PUT UNFORMATTED myXML.
  output CLOSE.
END.

PROCEDURE createPropath.
  OUTPUT TO VALUE(cWork + '.propath').
  ASSIGN myXML =
     '<?xml version=~"1.0~" encoding=~"UTF-8~" standalone=~"no~"?>' + {&CRLF1} +
     '<propath version=~"10.1C~" ' +
       'xmlns:xsi=~"http://www.w3.org/2001/XMLSchema-instance~" ' +
       'xsi:noNamespaceSchemaLocation=~"schema/propath.xsd~">' + {&CRLF1} +
     '<propathentry env=~"all~" kind=~"dir~" path=~"@~{WORK~}~"/>' + {&CRLF1} +
     '<propathentry env=~"all~" kind=~"src~" output=~"@~{ROOT~}~\rcode~" path=~"@~{ROOT~}~\src~"/>'  + {&CRLF1} +
     '<propathentry env=~"gui~" kind=~"con~" path=~"com.openedge.pdt.text.GUI_LIBRARIES~"/>'  + {&CRLF1} +
     '<propathentry env=~"tty~" kind=~"con~" path=~"com.openedge.pdt.text.TTY_LIBRARIES~"/>'  + {&CRLF1} +
     '<propathentry env=~"all~" kind=~"con~" path=~"com.openedge.pdt.text.DLC_PATHS~"/>'  + {&CRLF1} +
     '<propathentry env=~"web~" kind=~"con~" path=~"com.openedge.pdt.text.WEB_LIBRARIES~"/>'  + {&CRLF1} +
     '</propath>' .

  PUT UNFORMATTED myXML.
  output CLOSE.
END.

PROCEDURE createDBConnections.
  OUTPUT TO VALUE(cWork + '.dbconnection').
  ASSIGN myXML =
     '<?xml version=~"1.0~" encoding=~"UTF-8~" standalone=~"no~"?>' + {&CRLF1} +
     '<connection>'  + {&CRLF1} +
     '<connectionentry identifier=~"Moitaka10devel~"/>'  + {&CRLF1} +
     '</connection>' .

  PUT UNFORMATTED myXML.
  output CLOSE.
END.

QUIT.